/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CBuddy.h
  Info   : $Id: CBuddy.h,v 1.4 2006/01/20 22:57:53 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005, 2006 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CBuddy_h
#define CBuddy_h

// cxxx headers
#include <cxxx/CWindow.h>

// C++ headers

// optional headers

class CBuddy : public CWindow
{
private:
	enum
	{
		// State
		NO_BUTTON,
		UP_BUTTON,
		DOWN_BUTTON,
		// Geometry
		WIDTH = 11,
		HEIGHT = 16,
		// Message IDs
		IDTIMER = cxxx::msgid::MSGID_CBuddy
	};
public:
    CBuddy(CWindow* parent, unsigned msgid, bool show = true);
	virtual ~CBuddy();
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnButtonUp(XButtonEvent& e);
	virtual void OnMouseWheelUp(XButtonEvent& e);
	virtual void OnMouseWheelDown(XButtonEvent& e);
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	void SetPosition(int pos);
	void SetRange(int min, int max);
	void SetStep(int step);
	int GetPosition();
protected:
	unsigned mState;
	int mPos;
	int mMin;
	int mMax;
	int mStep;
	CIcon mIconUp;
	CIcon mIconDown;
	CTimer mTimer;
	void Increment();
	void Decrement();
};

inline void CBuddy::SetPosition(int pos)
{
	if (pos != mPos) {
		// range check
		if (pos < mMin) pos = mMin;
		if (pos > mMax) pos = mMax;
		// round
		pos = ((2 * pos + mStep) / (2 * mStep)) * mStep;
		mPos = pos;
	}
}

inline void CBuddy::SetRange(int min, int max)
{
	mMin = std::min(min, max);
	mMax = std::max(min, max);
	if (mPos < mMin) SetPosition(mMin);
	else if (mPos > mMax) SetPosition(mMax);
}

inline void CBuddy::SetStep(int step)
{
	assert(step >= 1);
	mStep = step;
}
		
inline int CBuddy::GetPosition()
{
	return mPos;
}

inline void CBuddy::Increment()
{
	SetPosition(mPos + mStep);
	SendMessage(SCROLL, mPos);
}

inline void CBuddy::Decrement()
{
	SetPosition(mPos - mStep);
	SendMessage(SCROLL, mPos);
}

#endif // CBuddy_h
