/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CCaret.h
  Info   : $Id: CCaret.h,v 1.5 2007/06/30 23:15:08 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CCaret_h
#define CCaret_h

// cxxx headers
#include <cxxx/CWindow.h>

class CCaret : public CWindow
{
	enum
	{
		IDTIMER = cxxx::msgid::MSGID_CCaret
	};
public:
	CCaret(CWindow* parent, bool show = false);
	~CCaret();
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	virtual void Recolor();
	void SetCaretHeight(unsigned h);
	void SetBlinkRate(unsigned rate);
	void ShowCaret();
	void HideCaret();
protected:
	CTimer mTimer;
	unsigned mBlinkRate;
	bool mVisible;
};

inline void CCaret::SetCaretHeight(unsigned h)
{
	Resize(1, h);
}

inline void CCaret::SetBlinkRate(unsigned rate)
{
	if (rate == 0) {
		HideCaret();
		mBlinkRate = 0;
	} else {
		assert(rate < 1000);
		mBlinkRate = rate;
	}
}

inline void CCaret::ShowCaret()
{
	Show();
	mVisible = true;
	if (mBlinkRate) mTimer.SetTime(mBlinkRate);
}

inline void CCaret::HideCaret()
{
	mTimer.Stop();
	mVisible = false;
	Hide();
}

#endif // CCaret_h
