/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CColorDialog.h
  Info   : $Id: CColorDialog.h,v 1.6 2006/01/28 16:16:37 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005, 2006 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CColorDialog_h
#define CColorDialog_h

// cxxx headers
#include <cxxx/CTopLevelWindow.h>
#include <cxxx/CSlider.h>
#include <cxxx/CNumEdit.h>
#include <cxxx/CToolButton.h>
#include <cxxx/CPushButton.h>

// C++ headers

// optional headers

class CColorDialog : public CTopLevelWindow
{
public:
	enum 
	{
		OK,
		CANCEL
	};
private:
	enum
	{
		IDRED = cxxx::msgid::MSGID_CColorDialog,
		IDGREEN,
		IDBLUE,
		IDRED_EDIT,
		IDGREEN_EDIT,
		IDBLUE_EDIT,
		IDCOLOR_PICKER,
		IDOK,
		IDCANCEL
	};
public:
    CColorDialog(CWindow* parent, const CColor& color);
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	virtual bool OnUpdate(unsigned msgid) const;
	virtual void OnButtonDown(XButtonEvent& e);
	virtual void OnKey(unsigned key, unsigned state);
	virtual void OnQuit();
	virtual ~CColorDialog();
	CColor GetColor();
protected:
	void DrawColorRectangle(CColor color);
	void OnColorPick();
	CSlider mRed;
	CSlider mGreen;
	CSlider mBlue;
	CNumEdit mRedEdit;
	CNumEdit mGreenEdit;
	CNumEdit mBlueEdit;
	CToolButton mColorPicker;
	CPushButton mButtonOK;
	CPushButton mButtonCancel;
	Cursor mCrossHairCursor;
	bool mPicking;
};

inline void CColorDialog::DrawColorRectangle(CColor color)
{
	GC gc = cxxx::cxGCManager->GetDefaultGC();
	XGCValues gcv;
	gcv.clip_mask = None;
	gcv.foreground = color.GetPixel();
	XChangeGC(cxxx::cxDpy, gc, GCClipMask | GCForeground, &gcv);
	SetGC(CGC::DEFAULT);
	FillRectangle(257, 12, 41, 41);
}

inline CColor CColorDialog::GetColor()
{
	return CColor(mRed.GetPosition(), mGreen.GetPosition(), mBlue.GetPosition());
}

#endif // CColorDialog_h
