/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CControl.h
  Info   : $Id: CControl.h,v 1.18 2007/06/30 23:15:08 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2004 - 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CControl_h
#define CControl_h

// cxxx headers
#include <cxxx/CDnDWindow.h>
#include <cxxx/CScrollBar.h>
#include <cxxx/CControlItem.h>

class CControl : public CDnDWindow
{
	enum
	{
		IDHSCROLL = cxxx::msgid::MSGID_CControl,
		IDVSCROLL,
		IDVIEW,
		IDTIMER
	};
protected:
	enum
	{
		ICON_WIDTH = 20,
		ICON_HEIGHT = 20,
		ICON_BORDER = 2,
		SCROLL_STEP = 20,
		BORDER_WIDTH = 2
	};
public:
	CControl(CWindow* parent, unsigned msgid, bool show = true);
	virtual ~CControl();
    virtual void OnConfigure(XConfigureEvent& e);
    virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
    virtual void OnFocus(bool in);
    virtual void OnDblClick(XButtonEvent& e);
    virtual void OnButtonUp(XButtonEvent& e);
	virtual void OnMouseWheelUp(XButtonEvent& e);
 	virtual void OnMouseWheelDown(XButtonEvent& e);
	virtual void OnKey(unsigned key, unsigned state);
	virtual void OnContext();
 	virtual void Recolor();
	void Update();
	void ClearList();
    unsigned GetItemCount() const;
    unsigned GetSelCount() const;
    void SelectAll();
    void DeselectAll();
    void InvertSelection();
	bool CanMultipleSelect() const;
    const std::vector<unsigned>& GetSelection();
	void Select(unsigned idx);
	void SetIconSize(unsigned w, unsigned h);
	bool IsSelected(unsigned idx);
protected:
	virtual void CalculateDocSize() = 0;
	virtual unsigned ItemFromPos(int x, int y) = 0;
	virtual void DrawItem(unsigned idx) = 0;
	virtual void ScrollIn(unsigned idx) = 0;
    void Layout();
	void Scroll(int x, int y);
    void Select(unsigned lo, unsigned hi);
    void Deselect(unsigned idx);
    void ToggleSelection(unsigned idx);
    void ScreenToWindow(int& x, int& y) const;
    void WindowToScreen(int& x, int& y) const;
	void DrawFrame();
	std::vector<CControlItem> mList;
    std::vector<unsigned> mSelection;
	std::vector<unsigned> mColumn;
	std::vector<unsigned> mRow;
	CHScrollBar mHScrollBar;
	CVScrollBar mVScrollBar;
	CTimer mScrollTimer;
	bool mMultipleSelection;
    int mXOffset;
	int mYOffset;
    unsigned mLastSel;
    unsigned mDocWidth;
    unsigned mDocHeight;
	unsigned mViewWidth;
	unsigned mViewHeight;
	unsigned mIconWidth;
	unsigned mIconHeight;
};

inline unsigned CControl::GetItemCount() const
{
	return mList.size();
}

inline void CControl::ClearList()
{
	mList.clear();
	mSelection.clear();
}

inline bool CControl::IsSelected(unsigned idx)
{
	assert(idx < mList.size());
	return mList[idx].IsSelected();
}

inline void CControl::SelectAll()
{
    assert(mMultipleSelection);
    for (unsigned i = 0; i < GetItemCount(); ++i) {
        if (!IsSelected(i)) {
            mList[i].Select(true);
            mSelection.push_back(i);
            DrawItem(i);
        }
    }
    DrawFrame();
}

inline void CControl::DeselectAll()
{
   	if (mSelection.empty()) return ;
    for (std::vector<unsigned>::iterator i = mSelection.begin(); i != mSelection.end(); ++i) {
        mList[*i].Select(false);
        DrawItem(*i);
    }
    mSelection.clear();
    DrawFrame();
}

inline void CControl::InvertSelection()
{
    assert(mMultipleSelection);
    mSelection.clear();
    for (unsigned i = 0; i < GetItemCount(); i++) {
	   	if (!IsSelected(i)) {
            mList[i].Select(true);
            mSelection.push_back(i);
        } else mList[i].Select(false);
        DrawItem(i);
	}
    DrawFrame();
}

inline void CControl::Select(unsigned idx)
{
    assert (idx < mList.size());
    if (IsSelected(idx)) return ;
    mList[idx].Select(true);
    mSelection.push_back(idx);
	mScrollTimer.SetTime(500);
    DrawItem(idx);
    DrawFrame();
}

inline void CControl::Deselect(unsigned idx)
{
    assert (idx < mList.size());
    if (!IsSelected(idx)) return;
    mList[idx].Select(false);
    std::vector<unsigned>::iterator it = find(mSelection.begin(), mSelection.end(), idx);
    if (it !=  mSelection.end()) mSelection.erase(it);
    DrawItem(idx);
    DrawFrame();
}

inline void CControl::Update()
{
	// force calculation of the document
	mDocHeight = 0;
	// do the work
	Layout();
	Redraw();
}

inline unsigned CControl::GetSelCount() const
{
	return mSelection.size();
}

inline bool CControl::CanMultipleSelect() const
{
	return mMultipleSelection;
}


inline const std::vector<unsigned>& CControl::GetSelection()
{
	return mSelection;
}


inline void CControl::ToggleSelection(unsigned idx)
{
	assert(idx < GetItemCount());
	if (IsSelected(idx)) Deselect(idx);
	else Select(idx);
}

inline void CControl::ScreenToWindow(int& x, int& y) const
{
	x += mXOffset * SCROLL_STEP - BORDER_WIDTH;
	y += mYOffset * SCROLL_STEP - BORDER_WIDTH;
}

inline void CControl::WindowToScreen(int& x, int& y) const
{
	x -= mXOffset * SCROLL_STEP - BORDER_WIDTH;
	y -= mYOffset * SCROLL_STEP - BORDER_WIDTH;
}

inline void CControl::DrawFrame()
{
    CWindow::DrawFrame(0, 0, GetWidth(), GetHeight(), SUNKEN_THICK, false);
	if (mHScrollBar.IsEnabled() && mVScrollBar.IsEnabled()) {
		SetGC(CGC::FRAME);
		FillRectangle(GetWidth() - mVScrollBar.GetWidth() - BORDER_WIDTH, GetHeight() - mHScrollBar.GetHeight() - BORDER_WIDTH, mVScrollBar.GetWidth(), mHScrollBar.GetHeight());
	}
}

inline void CControl::SetIconSize(unsigned w, unsigned h)
{
	mIconWidth = w + 2 * ICON_BORDER;
	mIconHeight = h + 2 * ICON_BORDER;
}

#endif // CControl_h
