/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CControlItem.h
  Info   : $Id: CControlItem.h,v 1.6 2006/09/10 15:39:03 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005, 2006 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CControlItem_h
#define CControlItem_h

class CControlItemInfo
{
public:
	CControlItemInfo(const std::string& text, bool sel)
	: mText(text),
	  mSelected(sel)
	{
	}
	virtual ~CControlItemInfo()
	{
	}
	std::string mText;
	unsigned mRefCount;
	bool mSelected;
};

class CControlItem
{
public:
	CControlItem();
	virtual ~CControlItem();
	
	CControlItem(const CControlItem& it);
	CControlItem& operator=(const CControlItem& it);
	bool operator==(const CControlItem& it) const;
	bool operator!=(const CControlItem& it) const;
	const std::string& GetText() const;
	bool IsSelected() const;
	void Select(bool sel);
protected:
	CControlItemInfo* mInfo;
};

inline CControlItem::CControlItem()
: mInfo(NULL)
{
}

inline CControlItem::~CControlItem()
{
	if (mInfo != NULL && --mInfo->mRefCount == 0) {
		delete mInfo;
		mInfo = NULL;
	}
}

inline CControlItem::CControlItem(const CControlItem& it)
: mInfo(it.mInfo)
{
	if (mInfo != NULL) ++mInfo->mRefCount;
}

inline CControlItem& CControlItem::operator=(const CControlItem& it)
{
	if (mInfo != it.mInfo) {
		if (mInfo != NULL && --mInfo->mRefCount == 0) delete mInfo;
		mInfo = it.mInfo;
		if (mInfo != NULL) ++mInfo->mRefCount;
	}
	return *this;
}

inline bool CControlItem::operator==(const CControlItem& it) const
{
	return mInfo == it.mInfo;
}

inline bool CControlItem::operator!=(const CControlItem& it) const
{
	return mInfo != it.mInfo;
}

inline const std::string& CControlItem::GetText() const
{
	return mInfo->mText;
}

inline bool CControlItem::IsSelected() const
{
	return mInfo->mSelected;
}

inline void CControlItem::Select(bool sel)
{
	mInfo->mSelected = sel;
}

#endif // CControlItem_h
