/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CDeskTopEntry.h
  Info   : $Id: CDeskTopEntry.h,v 1.7 2006/09/10 15:44:27 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2006 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CDesktopEntry_h
#define CDesktopEntry_h

// cxxx headers
#include <cxxx/Cxxx.h>

// C++ headers
#include <iosfwd>
#include <vector>

class CDeskTopEntry
{
public:
    typedef std::vector<CDeskTopEntry> TContainer;
	enum
	{
	    UNKNOWN,
	    APPLICATION,
	    LINK,
	    DEVICE,
	    MIME,
	    DIRECTORY,
	    SERVICE,
	    SERVICETYPE
	};

public:
	CDeskTopEntry();
	~CDeskTopEntry();
	CDeskTopEntry(const std::string& file);
	void Read(const std::string& file);
	const std::string& GetEncoding()
	{
		return mEncoding;
	}

	unsigned GetVersion()
	{
		return mVersion;
	}

    const std::string& GetName()
	{
        return mName;
	}

	unsigned GetType()
	{
		return mType;
	}

    const std::string& GetComment()
	{
		return mComment;
	}

	const std::string& GetExec()
	{
		return mExec;
	}

	TContainer& GetActions()
	{
		return mActions;
	}

    const std::string& GetIcon()
	{
		return mIcon;
	}

	const std::string& GetPath()
	{
		return mPath;
	}

	bool NeedTerm()
	{
		return mTerm;
	}

    const std::string& GetURL()
	{
		return mURL;
	}

protected:
    std::string mFile;
    std::string mEncoding;
	unsigned mVersion;
	std::string mName;
	unsigned mType;
	std::string mComment;
	TContainer mActions;
	std::string mIcon;
	std::string mPath;
	std::string mExec;
	bool mTerm;
	std::string mURL;
	void ReadAction(std::ifstream& ist, CDeskTopEntry* ent);
	std::string GetValue(const std::string& s);
	unsigned GetVersion(const std::string& s);
public:
};

#endif
