/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CDnDManager.h
  Info   : $Id: CDnDManager.h,v 1.4 2005/02/16 20:22:30 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2005 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CDnDManager_h
#define CDnDManager_h

// cxxx headers
#include <cxxx/CTimer.h>

// forward declarations
class CDnDWindow;
class CWindow;
class CMenuItem;

class CDnDManager : public CMessageTransceiver
{
private:
	enum
	{
		IDTIMEOUT = cxxx::msgid::MSGID_CDnDManager
	};
    enum
    {
        MIN_VERSION = 3,
        MAX_VERSION = 5
    };
public:
	CDnDManager();
	~CDnDManager();
    virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
    bool SetupDnD(CDnDWindow& w);
    bool HandleClientMessage(XClientMessageEvent& e);
    bool StartDrag(CDnDWindow& w, XMotionEvent& e, unsigned button);
    void StopDrag();
    void CancelDrag();
    bool OnMotion(CWindow& w, XMotionEvent& e)
    {
        if(((CDnDWindow*)&w == mDragSource)) {
            HandleMotionEvent(e);
            return true;
        } else return false;
    }
    void SetIgnoreRect(int x, int y, unsigned w, unsigned h)
    {
        mIgnoreRect.x = x;
        mIgnoreRect.y = y;
        mIgnoreRect.width = w;
        mIgnoreRect.height = h;
    }
    Cursor GetMoveCursor()
    {
        return mMoveCursor;
    }
    Cursor GetNotCursor()
    {
        return mNotCursor;
    }
    Cursor GetCopyCursor()
    {
        return mCopyCursor;
    }
    Cursor GetAskCursor()
    {
        return mAskCursor;
    }
    Cursor GetLinkCursor()
    {
        return mLinkCursor;
    }
    Cursor GetPrivateCursor()
    {
        return mPrivateCursor;
    }
private:
    bool CreateCursors();
    void UpdateCursor();
    bool RegisterAtoms();
    void HandleMotionEvent(XMotionEvent& e);
    void HandleNewPosition();
    Window FindTarget(XMotionEvent& e);
    CDnDWindow* GetTarget(int x, int y);
    void GetTypeList();
    bool SetTypeList();
    bool SetActionList();
    void SendEnter();
    void ReceiveEnter(XClientMessageEvent& e);
    void SendPosition();
    void ReceivePosition(XClientMessageEvent& e);
    void SendStatus();
    void ReceiveStatus(XClientMessageEvent& e);
    void SendLeave();
    void ReceiveLeave(XClientMessageEvent& e);
    void SendDrop();
    void ReceiveDrop(XClientMessageEvent& e);
    void SendFinished();
    void ReceiveFinished(XClientMessageEvent& e);
    Atom OnAskDialog();
public:
    static Atom DND_MOVE;
    static Atom DND_COPY;
    static Atom DND_LINK;
    static Atom DND_ASK;
    static Atom DND_PRIVATE;
    static Atom DND_SELECTION;
private:
    bool mInitialized;
    bool mWaitForStatus;
    bool mTargetResponds;
    bool mTypeListValid;
    bool mActionListValid;
    bool mFinalStatus;
    bool mDropRefused;
    unsigned mVersion;
    unsigned mEventMask;
    std::vector<Atom> mMimeList;
    std::vector<Atom> mActionList;
    std::vector<std::string> mDescriptionList;
    CTimer mTimeout;
    CDnDWindow* mDragSource;
    CDnDWindow* mDropTarget;
    Window mSourceXID;
    Window mTargetXID;
    Window mProxy;
    Atom mInitialAction;
    Atom mPerformedAction;
    XMotionEvent mActualPosition;
    XMotionEvent mLastPosition;
    XRectangle mIgnoreRect;
    Atom DND_AWARE;
    Atom DND_ENTER;
    Atom DND_POSITION;
    Atom DND_STATUS;
    Atom DND_LEAVE;
    Atom DND_DROP;
    Atom DND_FINISHED;
    Atom DND_PROXY;
    Atom DND_TYPES;
    Atom DND_ACTIONS;
    Atom DND_DESCRIPTIONS;
    Cursor mMoveCursor;
    Cursor mNotCursor;
    Cursor mCopyCursor;
    Cursor mAskCursor;
    Cursor mLinkCursor;
    Cursor mPrivateCursor;
    friend class CApplication;
};

#endif // CDnDManager_h

