/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CDnDWindow.h
  Info   : $Id: CDnDWindow.h,v 1.3 2005/02/16 20:22:31 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2005 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CDnDWindow_h
#define CDnDWindow_h

// cxxx headers
#include <cxxx/CWindow.h>
#include <cxxx/CDnDManager.h>


class CDnDWindow : public CWindow
{
private:
    enum
    {
        DRAG_THRESHOLD = 4
    };
public:
	CDnDWindow(CWindow* parent, unsigned msgid, bool show = true);
	virtual ~CDnDWindow();
    virtual void OnEvent(XEvent& e);
    virtual bool OnDragInit(unsigned button, Atom& action);
    virtual bool OnDrop(Atom action);
    virtual bool OnDropInit(std::vector<Atom> mime);
    virtual bool CanAcceptAction(Atom action);
    virtual bool CanPerformAction(Atom action);
	virtual Atom GetDnDAction(int x, int y, Atom Action);
	virtual void OnDropFinished();
    virtual Cursor GetDnDCursor(Atom action);
    virtual bool GetActionList(std::vector<Atom>& actions, std::vector<std::string>& desc);

protected:
    virtual void OnMotion(XMotionEvent& e);
    void SetupDnD();
    bool IsDragging();
    void StartDrag(XButtonEvent& e);
    void StopDrag(XButtonEvent& e);
    void CancelDrag();

    static Atom DND_MOVE;
    static Atom DND_COPY;
    static Atom DND_LINK;
    static Atom DND_ASK;
    static Atom DND_PRIVATE;
    static Atom DND_SELECTION;
private:
    static int mXDragStart;
    static int mYDragStart;
    unsigned mButton;
    bool mDragging;
};

inline bool CDnDWindow::IsDragging()
{
    return mDragging;
}

inline void CDnDWindow::StartDrag(XButtonEvent& e)
{
    if (IsDnDAware()) {
        mXDragStart = e.x;
        mYDragStart = e.y;
        mButton = e.button;
        unsigned mask = ButtonReleaseMask;
        switch (mButton) {
        case 1:
            mask |= Button1MotionMask;
            break;
        case 2:
            mask |= Button2MotionMask;
            break;
        case 3:
            mask |= Button3MotionMask;
            break;
        default:
            return;
        }
		XChangeActivePointerGrab(cxxx::cxDpy, mask, None, CurrentTime);
	}
}

inline void CDnDWindow::StopDrag(XButtonEvent& e)
{
    if (IsDnDAware() && IsDragging() && mButton == e.button) cxxx::cxDnDManager->StopDrag();
    else CancelDrag();
    mDragging = false;
}

inline void CDnDWindow::CancelDrag()
{
    if (IsDnDAware()) cxxx::cxDnDManager->CancelDrag();
    mDragging = false;
}

#endif // CDnDWindow_h

