/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CDropDownList.h
  Info   : $Id: CDropDownList.h,v 1.9 2007/06/30 23:15:08 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CDropDownList_h
#define CDropDownList_h

// cxxx headers
#include <cxxx/CWindow.h>
#include <cxxx/CListItem.h>

class CDropDownList : public CWindow
{
enum
{
	IDPOPUP = cxxx::msgid::MSGID_CDropDownList
};
typedef std::vector<std::string> TCont;
public:
	CDropDownList(CWindow* parent, unsigned msgid, bool show = true);
	~CDropDownList();
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnButtonUp(XButtonEvent& e);
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	virtual void Recolor();
	void AppendItem(const std::string& text);
	void DeleteAt(unsigned idx);
	unsigned GetItemCount();
	void Select(unsigned idx);
	void DropDown();
	const std::string& GetText();
	unsigned GetSelection();
	void Clear();
protected:
	CIcon mArrow;
	TCont mList;
	unsigned mSelection;
	bool mButtonDown;
};

inline void CDropDownList::AppendItem(const std::string& text)
{
	mList.push_back(text);
}

inline void CDropDownList::Clear()
{
	mList.clear();
}

inline unsigned CDropDownList::GetSelection()
{
	return mSelection;
}

inline void CDropDownList::DeleteAt(unsigned idx)
{
    TCont::iterator i = mList.begin() + idx;
    assert(i < mList.end());
	mList.erase(i);
}

inline unsigned CDropDownList::GetItemCount()
{
	return mList.size();
}

inline void CDropDownList::Select(unsigned idx)
{
	if (idx != -1u) {
		assert(idx < mList.size());
		mSelection = idx;
	}
	Redraw();
}

inline const std::string& CDropDownList::GetText()
{
    static std::string empty;
	if (mSelection == -1u) return empty;
    assert(mSelection < mList.size());
	return mList[mSelection];
}

#endif // CDropDownList_h
