/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CEdit.h
  Info   : $Id: CEdit.h,v 1.9 2007/06/30 23:15:08 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CEdit_h
#define CEdit_h

// cxxx headers
#include <cxxx/CCaret.h>

// forward declarations
class CCaret;

class CEdit : public CWindow
{
	enum
	{
		WIDTH = 100,
		BORDER_WIDTH = 2,
		IDTIMER = cxxx::msgid::MSGID_CCaret
	};
public:
	enum
	{
	    TOP		= 1 << 0,
	    BOTTOM	= 1 << 1,
	    YCENTER = 1 << 2,
	    LEFT	= 1 << 3,
	    RIGHT	= 1 << 4,
	    XCENTER	= 1 << 5,
		NUMERIC	= 1 << 6,
	    XJUSTIFY = LEFT | RIGHT | XCENTER,
	    YJUSTIFY = TOP | BOTTOM | YCENTER,
	    JUSTIFY = XJUSTIFY | YJUSTIFY
	};
	CEdit(CWindow* parent, bool show = true);
	~CEdit();
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnKey(unsigned key, unsigned mask);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnButtonUp(XButtonEvent& e);
	virtual void OnFocus(bool in);
	virtual void OnEvent(XEvent& e);
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	virtual void Enable(bool enable);
	virtual void Recolor();
	void SetText(const std::string& text);
	void SetOffset(unsigned off);
	void SetStyle(unsigned style);
	unsigned GetPosFromXpos(int xpos);
	void Select(unsigned min, unsigned max);
	const std::string& GetText() const;
	const std::string GetSelection();
	void ScrollText();
protected:
	void InsertChar(char c);
	void DeleteSelection();
	int GetXOffset();
	int GetYOffset();
	void SetCaretPos(unsigned pos);
protected:
	CCaret mCaret;
	unsigned mCaretPos;
	Cursor mCursor;
	unsigned mSelMax;
	unsigned mSelMin;
	unsigned mStyle;
	std::string mText;
	unsigned mTextWidth;
	CTimer mTimer;
	int mXOffset;
};

inline void CEdit::SetOffset(unsigned off)
{
	mXOffset = off;
	RedrawArea(BORDER_WIDTH, BORDER_WIDTH, mWidth - 2 * BORDER_WIDTH, mHeight - 2 * BORDER_WIDTH);
}

inline void CEdit::SetStyle(unsigned style)
{
	mStyle = style;
	SetCaretPos(mCaretPos);
	RedrawArea(BORDER_WIDTH, BORDER_WIDTH, mWidth - 2 * BORDER_WIDTH, mHeight - 2 * BORDER_WIDTH);
}

inline const std::string CEdit::GetSelection()
{
	if (mSelMin == mSelMax)
		return std::string("");
	else
		return mText.substr(mSelMin, mSelMax - mSelMin);
}

inline const std::string& CEdit::GetText() const
{
	return mText;
}

#endif // CEdit_h
