/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CFileListControl.h
  Info   : $Id: CFileListControl.h,v 1.22 2007/07/10 21:00:27 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CFileListControl_h
#define CFileListControl_h

// cxxx headers
#include <cxxx/CListControl.h>
#include <cxxx/CMagic.h>

// C headers
#include <sys/stat.h>

class CFileListControl : public CListControl
{
	enum
	{
		IDFILEWATCH = cxxx::msgid::MSGID_CFileListControl
	};
public:
	CFileListControl(CWindow* parent, unsigned msgid, bool show = true);
	~CFileListControl();
	void SetDirectory(const std::string& dir);
	std::string GetPath(unsigned idx) const;
	CFileListItem& GetItem(unsigned idx);
	void Reload();
 	void SetSortKey(unsigned key)
    {
        mFlags = key;
        Reload();
    }
	unsigned GetSortKey()
    {
        return mFlags;
    }
	unsigned GetIndex(const CFileListItem& item);
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	virtual void SetStyle(unsigned style, unsigned wico, unsigned hico);
	void AddFilter(TMagic typ);
	static bool CompNameA(const CFileListItem& i1, const CFileListItem& i2);
	static bool CompTimeA(const CFileListItem& i1, const CFileListItem& i2);
	static bool CompTypeA(const CFileListItem& i1, const CFileListItem& i2);
	static bool CompSizeA(const CFileListItem& i1, const CFileListItem& i2);
	static bool CompNameD(const CFileListItem& i1, const CFileListItem& i2);
	static bool CompTimeD(const CFileListItem& i1, const CFileListItem& i2);
	static bool CompTypeD(const CFileListItem& i1, const CFileListItem& i2);
	static bool CompSizeD(const CFileListItem& i1, const CFileListItem& i2);
	static bool CompNameAD(const CFileListItem& i1, const CFileListItem& i2);
	static bool CompTimeAD(const CFileListItem& i1, const CFileListItem& i2);
	static bool CompSizeAD(const CFileListItem& i1, const CFileListItem& i2);
	static bool CompNameDD(const CFileListItem& i1, const CFileListItem& i2);
	static bool CompTimeDD(const CFileListItem& i1, const CFileListItem& i2);
	static bool CompSizeDD(const CFileListItem& i1, const CFileListItem& i2);
protected:
    virtual void ReadDirectory();
    virtual void DrawItem(unsigned idx);
	virtual void Append(const std::string& path, const std::string& name, const struct stat& st);
    void Sort();
protected:
	unsigned mFlags;
	std::string mCurrentDir;
    CMagic mMagic;
    CIcon mLinkOverlay;
	CIcon mZipOverlay;
    bool mWriteAccess;
    time_t mFileTime;
    CTimer mFileWatch;
    std::vector<TMagic> mFilter;
	std::string mMimeDir;
};

inline void CFileListControl::AddFilter(TMagic typ)
{
	if (typ == DELETE) mFilter.clear();
	else mFilter.push_back(typ);
}

inline void CFileListControl::Reload()
{
	std::vector<std::string> sel;
	if (!mSelection.empty()) {
		for (std::vector<unsigned>::iterator i = mSelection.begin(); i < mSelection.end(); ++i) sel.push_back(GetItem(*i).GetText());
	}
	ClearList();
	ReadDirectory();
	if (!sel.empty()) {
		for (std::vector<std::string>::iterator i = sel.begin(); i < sel.end(); ++i) {
			for (unsigned j = 0; j < mList.size(); ++j) {
				if (GetItem(j).GetText() == *i) {
					GetItem(j).Select(true);
					mSelection.push_back(j);
				}
			}
		}
	}
	Update();
}

inline std::string CFileListControl::GetPath(unsigned idx) const
{
	std::string ret(mCurrentDir);
	if (idx < mList.size())  {
		if (ret[ret.size() - 1] != '/') ret += '/';
		ret += ((const CFileListItem&)mList[idx]).GetText();
	}
	return ret;
}

inline CFileListItem& CFileListControl::GetItem(unsigned idx)
{
	return (CFileListItem&)mList[idx];
}

inline bool CFileListControl::CompNameA(const CFileListItem& i1, const CFileListItem& i2)
{
	return cxxx::cxLocale(i1.GetText(), i2.GetText());
}

inline bool CFileListControl::CompTimeA(const CFileListItem& i1, const CFileListItem& i2)
{
	int cmp = i1.GetTime() - i2.GetTime();
	if (cmp) return cmp < 0;
	return CompNameA(i1, i2);
}

inline bool CFileListControl::CompTypeA(const CFileListItem& i1, const CFileListItem& i2)
{
	int cmp = S_ISDIR(i2.GetMode()) - S_ISDIR(i1.GetMode());
	if (cmp) return cmp < 0;
	cmp = i1.GetType() - i2.GetType();
	if (cmp) return cmp < 0;
	return CompNameA(i1, i2);
}

inline bool CFileListControl::CompSizeA(const CFileListItem& i1, const CFileListItem& i2)
{
	int cmp = i1.GetSize() - i2.GetSize();
	if (cmp) return cmp < 0;
	return CompNameA(i1, i2);
}

inline bool CFileListControl::CompNameD(const CFileListItem& i1, const CFileListItem& i2)
{
	return CompNameA(i2, i1);
}

inline bool CFileListControl::CompTimeD(const CFileListItem& i1, const CFileListItem& i2)
{
	return CompTimeA(i2, i1);
}

inline bool CFileListControl::CompTypeD(const CFileListItem& i1, const CFileListItem& i2)
{
	int cmp = S_ISDIR(i1.GetMode()) - S_ISDIR(i2.GetMode());
	if (cmp) return cmp < 0;
	cmp = i1.GetType() - i2.GetType();
	if (cmp) return cmp < 0;
	return CompNameA(i2, i1);
}

inline bool CFileListControl::CompSizeD(const CFileListItem& i2, const CFileListItem& i1)
{
	int cmp = i1.GetSize() - i2.GetSize();
	if (cmp) return cmp < 0;
	return CompNameA(i1, i2);
}

inline bool CFileListControl::CompNameAD(const CFileListItem& i1, const CFileListItem& i2)
{
	int cmp = S_ISDIR(i2.GetMode()) - S_ISDIR(i1.GetMode());
	if (cmp) return cmp < 0;
	return CompNameA(i1, i2);
}

inline bool CFileListControl::CompTimeAD(const CFileListItem& i1, const CFileListItem& i2)
{
	int cmp = S_ISDIR(i2.GetMode()) - S_ISDIR(i1.GetMode());
	if (cmp) return cmp < 0;
	return CompTimeA(i1, i2);
}

inline bool CFileListControl::CompSizeAD(const CFileListItem& i1, const CFileListItem& i2)
{
	int cmp = S_ISDIR(i2.GetMode()) - S_ISDIR(i1.GetMode());
	if (cmp) return cmp < 0;
	return CompSizeA(i1, i2);
}

inline bool CFileListControl::CompNameDD(const CFileListItem& i1, const CFileListItem& i2)
{
	int cmp = S_ISDIR(i2.GetMode()) - S_ISDIR(i1.GetMode());
	if (cmp) return cmp < 0;
	return CompNameD(i1, i2);
}

inline bool CFileListControl::CompTimeDD(const CFileListItem& i1, const CFileListItem& i2)
{
	int cmp = S_ISDIR(i2.GetMode()) - S_ISDIR(i1.GetMode());
	if (cmp) return cmp < 0;
	return CompTimeD(i1, i2);
}

inline bool CFileListControl::CompSizeDD(const CFileListItem& i1, const CFileListItem& i2)
{
	int cmp = S_ISDIR(i2.GetMode()) - S_ISDIR(i1.GetMode());
	if (cmp) return cmp < 0;
	return CompSizeD(i1, i2);
}

#endif // CFileListControl_h
