/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CFileTree.h
  Info   : $Id: CFileTree.h,v 1.15 2005/08/14 15:43:15 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2005 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CFileTree_h
#define CFileTree_h

// cxxx headers
#include <cxxx/CTreeControl.h>

class CFileTree : public CTreeControl
{
public:
	enum // return values of GetStatus
	{
	    NOSUBDIR,
	    SUBDIRS,
	    ISLOCKED
	};
	CFileTree(CWindow *parent, unsigned msgid, bool show = true);
	virtual void Open(unsigned idx);
	void SetDirectory(const std::string& dir);
	void SetSortKey(unsigned key);
	void Reload();
	std::string GetPath(unsigned idx);
	virtual void Append(CTreeItem& parent, const std::string& closed, const std::string& opened, const std::string& text);
protected:
	virtual void DrawItem(unsigned idx);
	std::string GetPath(const CTreeItem& item) const;
	void ReadDir(CTreeItem& item);
	unsigned GetStatus(const std::string& dir);
	static bool CompNameA(const CTreeItem& i1, const CTreeItem& i2);
	static bool CompNameD(const CTreeItem& i1, const CTreeItem& i2);
	void Sort(CTreeItem& item);
	unsigned mFlags;
	CIcon mLockIcon;
};

inline void CFileTree::Reload()
{
	unsigned idx(0);
	if (GetSelCount() != 0) idx = GetSelection().front();
	std::string sel;
	bool open(false);
	if (idx < GetItemCount()) {
		sel = GetPath(GetItem(idx));
		open = GetItem(idx).IsOpened();
	} else sel = "/";
	mTree.GetChildren().clear();
	SetDirectory(sel);
	if (open) Open(GetSelection().front());
}

inline void CFileTree::SetSortKey(unsigned key)
{
	mFlags = key;
	Reload();
}

inline bool CFileTree::CompNameA(const CTreeItem& i1, const CTreeItem& i2)
{
	return cxxx::cxLocale(i1.GetText(), i2.GetText());
}

inline bool CFileTree::CompNameD(const CTreeItem& i1, const CTreeItem& i2)
{
	return CompNameA(i2, i1);
}

inline std::string CFileTree::GetPath(unsigned idx)
{
	if (idx < mList.size()) return GetPath(GetItem(idx));
	assert (!mList.empty());
	return mTree.GetText();
}

#endif // CFileTree_h
