/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CFont.h
  Info   : $Id: CFont.h,v 1.10 2005/12/30 17:34:05 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2005 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CFont_h
#define CFont_h

// cxxx headers
#include <cxxx/Cxxx.h>

// C++ headers
#include <vector>

// optional headers
#ifdef HAVE_LIBPNG
	#include <png.h>
#endif
#ifdef AA_TEXT_SUPPORT
  #include <ft2build.h>
  #include FT_FREETYPE_H
  #include FT_GLYPH_H
  #include <X11/extensions/Xrender.h>
  #include <fontconfig/fontconfig.h>
#endif // AA_TEXT_SUPPORT

typedef enum
{
	THIN 		= 0,
	EXTRALIGHT 	= 40,
	LIGHT		= 50,
	BOOK		= 75,
	REGULAR		= 80,
	MEDIUM		= 100,
	DEMIBOLD	= 180,
	BOLD		= 200,
	EXTRABOLD	= 205,
	BLACK		= 210
} TFontWeight;

typedef enum
{
	ULTRACONDENSED	= 50,
	EXTRACONDENSED	= 63,
	CONDENSED		= 75,
	SEMICONDENSED	= 87,
	NORMAL			= 100,
	SEMIEXPANDED	= 113,
	EXPANDED		= 125,
	EXTRAEXPANDED	= 150,
	ULTRAEXPANDED	= 200
} TFontWidth;

typedef enum
{
	ANY,
	ROMAN,
	ITALIC,
	OBLIQUE
} TFontSlant;

struct CFontDescriptor
{
	std::string mFoundry;
	std::string mFamily;
	TFontWeight mWeight;
	TFontSlant mSlant;
	TFontWidth mWidth;
	int mSize;
	unsigned mEncoding;
};

class CFont
{
public:
#ifdef AA_TEXT_SUPPORT
    struct CGlyph
    {
        XGlyphInfo mInfo;
        Glyph mID;

        CGlyph()
        : mID(-1u)
        {}
    };
#endif // AA_TEXT_SUPPORT

	CFont();
	~CFont();
	
    bool LoadFont(const std::string& xlfd);
#ifdef AA_TEXT_SUPPORT
    const CGlyph& GetGlyph(char c);
    GlyphSet GetGlyphset();
#endif // AA_TEXT_SUPPORT
    unsigned GetHeight();
    unsigned GetAscent();
    unsigned GetDescent();
	XFontStruct* GetCore();
	bool IsAntialiased();
	static void ListFonts(std::vector<CFontDescriptor>& list);
	void Clear();
	static CFontDescriptor ParseXlfd(const std::string& xlfd);
	static std::string GetStyle(const CFontDescriptor& fd);
private:
	std::string MatchCoreFont(const CFontDescriptor& fd);
	static void ListAAFonts(std::vector<CFontDescriptor>& l);
	static void ListCoreFonts(std::vector<CFontDescriptor>& l);
	bool LoadAAFont(const CFontDescriptor& fd);
	bool LoadCoreFont(const CFontDescriptor& fd);
#ifdef AA_TEXT_SUPPORT
	FcPattern* MatchAAFont(const CFontDescriptor& fd);
	static FcCharSet* CreateCharSet(const std::string& lang);
    void LoadGlyph(char c);
    int GetFloor(FT_F26Dot6 i);
    int GetRound(FT_F26Dot6 i);
    int GetCeil(FT_F26Dot6 i);

    static FT_Library mFTLib;
    FT_Face mFace;
    CGlyph mGlyphs[256];
    unsigned mGlyphCount;
    GlyphSet mGlyphset;
#endif //AA_TEXT_SUPPORT
	XFontStruct* mCore;
	unsigned mAscent;
	unsigned mDescent;
};

#ifdef AA_TEXT_SUPPORT
inline const CFont::CGlyph& CFont::GetGlyph(char c)
{
    if (mGlyphs[c & 0xff].mID == -1u) LoadGlyph(c);
    return mGlyphs[c & 0xff];
}

inline GlyphSet CFont::GetGlyphset()
{
    return mGlyphset;
}

inline int CFont::GetFloor(FT_F26Dot6 i)
{
    return i >> 6;
}

inline int CFont::GetRound(FT_F26Dot6 i)
{
    return (i + 32) >> 6;
}

inline int CFont::GetCeil(FT_F26Dot6 i)
{
    return (i + 63) >> 6;
}
#endif //AA_TEXT_SUPPORT

inline unsigned CFont::GetHeight()
{
    return GetAscent() + GetDescent();
}

inline unsigned CFont::GetAscent()
{
    return mAscent;
}

inline unsigned CFont::GetDescent()
{
    return mDescent;
}

inline XFontStruct* CFont::GetCore()
{
	return mCore;
}

inline bool CFont::IsAntialiased()
{
#ifdef AA_TEXT_SUPPORT
	return mFace != NULL;
#else
	return false;
#endif // AA_TEXT_SUPPORT
}

#endif // CFont_h
