/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CFontSheet.h
  Info   : $Id: CFontSheet.h,v 1.12 2005/11/27 17:53:06 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CFontSheet_h
#define CFontSheet_h

// cxxx headers
#include <cxxx/CPushButton.h>
#include <cxxx/CDropDownList.h>
#include <cxxx/CLabel.h>

// C++ headers
#include <map>

// optional headers

typedef std::map<std::string, std::map<std::string, unsigned> > CFontList;

struct CFontStyleNames
{
	const char* mOriginal;
	unsigned mTranslation;
};

class CFontSheet : public CWindow
{
private:
	enum
	{
		// message ids
		IDTEXTFAMILY = cxxx::msgid::MSGID_CFontSheet,
		IDTEXTSTYLE,
		IDTEXTSIZE,
		IDTITLE,
		IDTITLEFAMILY,
		IDTITLESTYLE,
		IDTITLESIZE,
		// array sizes
		WEIGHTCOUNT = 8,
		SLANTCOUNT = 2,
		WIDTHCOUNT = 8
	};
public:
    CFontSheet(CWindow* parent);
	virtual ~CFontSheet();
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	bool DoOK();
protected:
	CFontList mFontList;
	CDropDownList mTextFontFamily;
	CDropDownList mTitleFontFamily;
	CDropDownList mTextFontStyle;
	CDropDownList mTitleFontStyle;
	CDropDownList mTextFontSize;
	CDropDownList mTitleFontSize;
	CLabel mTextFontPreview;
	CLabel mTitleFontPreview;
	CFont mTextFont;
	CFont mTitleFont;
	//std::string mXlfd;
protected:
	std::string ComposeXlfd(bool title);
	void FillStyle(CDropDownList& d, const std::map<std::string, unsigned>& m, const std::string& preset);
	void FillSize(CDropDownList& d, unsigned s, unsigned preset);
	void CreateFontList();
	std::string Localize(const std::string& s);
	std::string DeLocalize(const std::string& s);
	static const CFontStyleNames mWeightNames[WEIGHTCOUNT];
	static const CFontStyleNames mSlantNames[SLANTCOUNT];
	static const CFontStyleNames mWidthNames[WIDTHCOUNT];
};

inline std::string CFontSheet::ComposeXlfd(bool title)
{
	std::string fnt;
	if (title) {
		fnt = mTitleFontFamily.GetText();
		fnt += '-';
		fnt += DeLocalize(mTitleFontStyle.GetText());
		fnt += '-';
		fnt += mTitleFontSize.GetText();
	} else {
		fnt = mTextFontFamily.GetText();
		fnt += '-';
		fnt += DeLocalize(mTextFontStyle.GetText());
		fnt += '-';
		fnt += mTextFontSize.GetText();
	}
	return fnt;
}

#endif // CFontSheet_h
