/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CGC.h
  Info   : $Id: CGC.h,v 1.9 2006/12/09 18:33:48 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2006 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CGC_h
#define CGC_h

// cxxx headers
#include <cxxx/CColor.h>
#include <cxxx/CFont.h>

// C++ headers

class CGC
{
public:
	enum {
        BLACK = CColor::BLACK,
		WHITE = CColor::WHITE,
		FRAME = CColor::FRAME,
		DARK = CColor::DARK,
		BRIGHT = CColor::BRIGHT,
        TITLE = CColor::TITLE,
        TOOLTIP = CColor::TOOLTIP,
		SCROLL = CColor::SCROLL,
		BACKGND = CColor::BACKGND,
        DOTTED = CColor::COLORCOUNT,
		STIPPLED,
		ALPHA8 = STIPPLED,
#ifdef HAVE_LIBXRENDER
		ARGB32,
#endif // HAVE_LIBXRENDER
		DEFAULT,
		GCCOUNT
    };
public:
	CGC();
	~CGC();
    GC GetGC();
    CColor GetColor();
    void CreateGCs(XID id);
    void DestroyGCs();
    GC GetPictureGC();
    GC GetDefaultGC();
#ifdef HAVE_LIBXRENDER
	GC GetAlphaGC();
	GC GetArgbGC();
    Picture GetPicture();
	Picture GetGrayOut();
    XRenderPictFormat* GetAlphaFormat();
    XRenderPictFormat* GetWindowFormat();
	XRenderPictFormat* GetImageFormat();
#else // HAVE_LIBXRENDER
	GC GetStippledGC();
#endif // HAVE_LIBXRENDER

private:
    void SetGC(unsigned idx);
    void SetColor(unsigned idx);
 	void SetTextColor(unsigned idx);

    GC mGC[GCCOUNT];
    GC mCurrentGC;
    CColor mCurrentColor;
#ifdef HAVE_LIBXRENDER
    Picture mPict[CColor::COLORCOUNT];
    Picture mCurrentPicture;
	Picture mGrayedOut;
    XRenderPictFormat* mAlphaFormat;
    XRenderPictFormat* mWindowFormat;
	XRenderPictFormat* mImageFormat;
#endif // HAVE_LIBXRENDER

friend class CWindow;
};

inline GC CGC::GetGC()
{
	return mCurrentGC;
}

inline void CGC::SetGC(unsigned idx)
{
	assert(idx < GCCOUNT);
	mCurrentGC = mGC[idx];
}


inline CColor CGC::GetColor()
{
	return mCurrentColor;
}

inline void CGC::SetColor(unsigned idx)
{
	mCurrentColor = CColor::GetStdColor(idx);
	mCurrentGC = mGC[idx];
#ifdef HAVE_LIBXRENDER
	mCurrentPicture = mPict[idx];
#endif // HAVE_LIBXRENDER
}

#ifdef HAVE_LIBXRENDER
inline Picture CGC::GetPicture()
{
	assert(mCurrentPicture != None);
	return mCurrentPicture;
}

inline Picture CGC::GetGrayOut()
{
	assert(mGrayedOut != None);
	return mGrayedOut;
}
#endif // HAVE_LIBXRENDER

inline void CGC::SetTextColor(unsigned idx)
{
	if (CColor::GetStdColor(idx).GetLuminance() >= 0x80) SetColor(BLACK);
	else SetColor(WHITE);
}

inline GC CGC::GetPictureGC()
{
    assert(mGC[SCROLL] != NULL);
	return mGC[SCROLL];
}

inline GC CGC::GetDefaultGC()
{
	return mGC[DEFAULT];
}

#ifdef HAVE_LIBXRENDER

inline GC CGC::GetAlphaGC()
{
	assert(mGC[ALPHA8] != NULL);
	return mGC[ALPHA8];
}

inline GC CGC::GetArgbGC()
{
	assert (mGC[ARGB32] != NULL);
	return mGC[ARGB32];
}

inline XRenderPictFormat* CGC::GetAlphaFormat()
{
	assert(mAlphaFormat != NULL);
	return mAlphaFormat;
}

inline XRenderPictFormat* CGC::GetWindowFormat()
{
	assert(mWindowFormat != NULL);
	return mWindowFormat;
}

inline XRenderPictFormat* CGC::GetImageFormat()
{
	assert(mImageFormat != NULL);
	return mImageFormat;
}

#else // HAVE_LIBXRENDER

inline GC CGC::GetStippledGC()
{
	assert(mGC[STIPPLED] != NULL);
	return mGC[STIPPLED];
}

#endif // HAVE_LIBXRENDER

#endif // CGC_h
