/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CI18n.h
  Info   : $Id: CI18n.h,v 1.12 2005/08/14 15:43:15 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CI18n_h
#define CI18n_h

#include <vector>
#include <string>
#include <cassert>
#include <locale>

class CI18n
{
public:
	enum
	{
		SUCCESS,
		NOT_EMPTY,
		NO_LOCALE_DIR,
		EMPTY_LOCALE_DIR,
		NO_LOCALE_NAME,
		NO_FILE,
		FILE_TYPE,
		FILE_ERROR,
		C_ONLY,
		ERROR_COUNT
	};		
public:
	CI18n();
	~CI18n() {};
	void AddSearchPath(const std::string& path);
	void ClearSearchPath();
	unsigned Open(const std::string& name, const std::locale& loc);
	const char* GetText(unsigned idx);
	void Close();
	static const char* GetErrorText(unsigned err);
private:
	std::vector<std::string> mList;
	std::vector<std::string> mDirs;
	static const char* mErrorText[ERROR_COUNT];
};

inline CI18n::CI18n()
{
	mDirs.push_back("/usr/share/locale");
	mDirs.push_back("/usr/local/share/locale");
}

inline void CI18n::AddSearchPath(const std::string& path)
{
	mDirs.insert(mDirs.begin(), path);
}

inline void CI18n::ClearSearchPath()
{
	mDirs.clear();
}

inline const char* CI18n::GetText(unsigned idx)
{
	if (idx < mList.size()) return mList[idx].c_str();
	else return "CI18n: Index out of range.";
}

inline void CI18n::Close()
{
	mList.clear();
}

inline const char* CI18n::GetErrorText(unsigned err)
{
	return mErrorText[err];
}

# endif // CI18n_h
