/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CIcon.h
  Info   : $Id: CIcon.h,v 1.9 2005/07/26 16:55:43 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2005 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CIcon_h
#define CIcon_h

// cxxx headers
#include <cxxx/Cxxx.h>

// C++ headers
#include <cassert>
#include <map>


class CIconInfo
{
public:
    Pixmap mPixmap;
    Pixmap mClipmask;
    unsigned mWidth;
    unsigned mHeight;
    unsigned mRefCount;
    unsigned mType;

    CIconInfo()
	: mPixmap(None),
	mClipmask(None),
	mWidth(0),
	mHeight(0),
	mRefCount(0),
	mType(0)
    {}

    ~CIconInfo();
};

class CIcon
{
public:

    CIcon()
    : mInfo(NULL)
    {}

    CIcon(const CIcon& ico)
    : mInfo(ico.mInfo)
    {
        Attach();
    }
	
	CIcon(const std::string& fn)
	: mInfo(NULL)
	{
		LoadFile(fn);
	}

    ~CIcon()
    {
        Detach();
    }

    CIcon& operator=(const CIcon& ico);
    void LoadFile(const std::string& fn);
	void LoadScaled(const std::string& fn, unsigned w, unsigned h, unsigned flag);
    void Clear();
    bool IsValid() const;
    unsigned GetWidth() const;
    unsigned GetHeight() const;
	Pixmap GetClipmask() const;
    Pixmap GetPixmap() const;
    unsigned GetType() const;
	static std::string FindFileName(const std::string& fn);
private:
    void Attach();
    void Detach();
    CIconInfo* mInfo;
	static bool mCanCleanup;
    static std::map<std::string, CIcon> mIconContainer;
};

inline CIcon& CIcon::operator=(const CIcon& ico)
{
	if (mInfo == ico.mInfo) return *this;
	Detach();
	mInfo = ico.mInfo;
	Attach();
	return *this;
}

inline void CIcon::Clear()
{
	Detach();
}

inline bool CIcon::IsValid() const
{
	return mInfo != NULL;
}

inline unsigned CIcon::GetWidth() const
{
	if (IsValid()) return mInfo->mWidth;
	else return 0;
}

inline unsigned CIcon::GetHeight() const
{
	if (IsValid()) return mInfo->mHeight;
	else return 0;
}

inline Pixmap CIcon::GetClipmask() const
{
	if (IsValid()) return mInfo->mClipmask;
	else return None;
}

inline Pixmap CIcon::GetPixmap() const
{
	if (IsValid()) return mInfo->mPixmap;
	else return None;
}

inline unsigned CIcon::GetType() const
{
	if (IsValid()) return mInfo->mType;
	else return 0;
}

inline void CIcon::Attach()
{
	if (mInfo != NULL) ++mInfo->mRefCount;
}

inline void CIcon::Detach()
{
	if (mInfo != NULL)
	{
		assert(mInfo->mRefCount != 0);
		if (--mInfo->mRefCount <= 1) mCanCleanup = true;
		if (mInfo->mRefCount == 0) delete mInfo;		
		mInfo = NULL;
	}
}

#endif // CIcon_h
