/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CListControl.h
  Info   : $Id: CListControl.h,v 1.21 2007/07/31 17:39:02 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CListControl_h
#define CListControl_h

// cxxx headers
#include <cxxx/CListItem.h>
#include <cxxx/CControl.h>

class CListControl : public CControl
{
protected:
    enum
    {
        TEXT_BORDER = 2,
        MIN_ITEM_WIDTH = 40,
		INTER_SPACE = 6,
		ICON_ITEM_WIDTH = 80
    };
public:
	enum
	{
		MULTI_COLUMN_VIEW = 0,
		ICON_VIEW
	};
public:
	CListControl(CWindow* parent, unsigned msgid, bool show = true);
    virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnRButtonDown(XButtonEvent& e);
	virtual void OnKey(unsigned key, unsigned state);
	virtual void SetStyle(unsigned style, unsigned wico, unsigned hico);
	CListItem& GetItem(unsigned idx);
protected:
	virtual void CalculateDocSize();
	virtual void DrawItem(unsigned idx);
    virtual void ScrollIn(unsigned idx);
	unsigned ItemFromPos(int x, int y);
    unsigned GetColumn(int x);
    unsigned GetRow(int y);
	void GetBounds(int& x, int& y, unsigned& w, unsigned& h);
protected:
	unsigned mStyle;
};

inline unsigned CListControl::ItemFromPos(int x, int y)
{
	ScreenToWindow(x, y);
	unsigned row = GetRow(y);
	if (row >= mRow.size()) return -1u;
	unsigned col = GetColumn(x);
	if (col >= mColumn.size()) return -1u;
	unsigned idx;	
	if (mStyle == MULTI_COLUMN_VIEW) idx = row + mRow.size() * col;
	else idx = col + mColumn.size() * row; 
	if (idx >= mList.size()) return -1u;
	// inside the item's square, see if it's hit
	if (mStyle == MULTI_COLUMN_VIEW) {
		int xh;
		if (col) xh = mColumn[--col];
		else xh = 0;
		xh += GetItem(idx).GetWidth();
		if(x > xh) return -1u;
		else return idx;
	} else {
		int xh = mColumn[col];
		int xl;
		if (col) xl = mColumn[--col];
		else xl = 0;
		int w = xh - xl;
		int y0;
		if (row) y0 = mRow[--row];
		else y0 = 0;
		if (y - y0 > (int)GetItem(idx).GetHeight()) return -1u;
		y0 = y - y0 - mIconHeight;
		if (y0 < 0) {
			xh = xl + (w + mIconWidth) / 2;
			xl += (w - mIconWidth) / 2;				
		} else {
			y0 /= GetTextHeight() + 2 * TEXT_BORDER;
			xh = xl + (w + GetItem(idx).GetTextHint(y0).mWidth) / 2;
			xl += (w - GetItem(idx).GetTextHint(y0).mWidth) / 2;
		}
		if (x < xl || x > xh) return -1u;
		else return idx;
	}
}

inline unsigned CListControl::GetColumn(int x)
{
	if (x <= 0) return 0;
	std::vector<unsigned>::iterator i = std::lower_bound(mColumn.begin(), mColumn.end(), (unsigned) x);
	if (i == mColumn.end()) return -1u;
	return i - mColumn.begin();
}

inline unsigned CListControl::GetRow(int y)
{
	if (y <= 0) return 0;
	std::vector<unsigned>::iterator i = std::lower_bound(mRow.begin(), mRow.end(), (unsigned) y);
	if (i == mRow.end()) return -1u;
	return i - mRow.begin();
}

inline void CListControl::ScrollIn(unsigned idx)
{
	unsigned col;
	unsigned row;
	// translate index to row and column
	if (mStyle == MULTI_COLUMN_VIEW) {
		col = idx / mRow.size();
		row = idx % mRow.size();
	} else {
		col = idx % mColumn.size();
		row = idx / mColumn.size();
	}
	int xi;
	if (col) xi = mColumn[col - 1];
	else xi = 0;
	xi += GetItem(idx).GetWidth();
	int yi;
	if (row) yi = mRow[row - 1];
	else yi = 0;
	yi += GetItem(idx).GetHeight();
	int xw = BORDER_WIDTH + mViewWidth;
	int yw = BORDER_WIDTH + mViewHeight;
	int scrlx = 0;
	int scrly = 0;
	WindowToScreen(xi, yi);
	if (yi > yw) scrly = (yi - yw + SCROLL_STEP - 1) / SCROLL_STEP;
	yi -= GetItem(idx).GetHeight();
	yw -= mViewHeight;
	if (yi < yw) scrly += (yi - yw - SCROLL_STEP + 1) / SCROLL_STEP;
	if (xi > xw) scrlx = (xi - xw + SCROLL_STEP - 1) / SCROLL_STEP;
	xi -= GetItem(idx).GetWidth();
	xw -= mViewWidth;
	if (xi < xw) scrlx += (xi - xw - SCROLL_STEP + 1) / SCROLL_STEP;
	if (scrlx != 0) mHScrollBar.SetPosition(mHScrollBar.GetPosition() + scrlx);
	if (scrly != 0) mVScrollBar.SetPosition(mVScrollBar.GetPosition() + scrly);
}

inline CListItem& CListControl::GetItem(unsigned idx)
{
	assert(idx < mList.size());
	return (CListItem&)mList[idx];
}

#endif // CListControl_h
