/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CMagic.h
  Info   : $Id: CMagic.h,v 1.25 2007/05/20 17:42:15 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CMagic_h
#define CMagic_h

// cxxx headers
#include <cxxx/Cxxx.h>
#include <cxxx/TMagic.h>

// C++ headers
#include <istream>
#include <cstring>
#include <algorithm>
#include <fstream>

// C headers
#include <sys/types.h>
#include <sys/stat.h>

class CMagic
{
public:
	struct CMagInfo
	{
		const char* mIcon;
		const char* mMime;
		unsigned mDescription;
	};		
public:

	struct CGlob
	{
		const char* mPattern;
		TMagic mType;
	};

	CMagic();
	~CMagic();
	TMagic GetFileType(const std::string& file);
	TMagic GetFileType(std::filebuf& fbuf, const struct stat& st, const std::string& fn);
	static const char* GetDescription(TMagic type);
    static const char* GetMimeType(TMagic type);
	static const char* GetIcon(TMagic type);
protected:
	enum
	{
		ENDINGSIZE = 122,
		NAMESIZE = 58,
		MAGIC0SIZE = 233,
		OPENOFFICESIZE = 9,
		KOFFICESIZE = 10
	};

	static bool Less(const CGlob& g1, const CGlob& g2);
	static bool LessMag(const CGlob& g1, const CGlob& g2);

	TMagic GetTypeByMagic();
	TMagic GetTypeByGlobbing();
	TMagic GetSpecialType(mode_t m);
	TMagic GetTextType();
	void ResetFileTimes();
	std::string GetSuffix();
	std::string mPath;
    std::istream mStream;
	std::filebuf mFilebuffer;
	std::ios::pos_type mOffset;
	unsigned mStreamSize;
	struct stat mStatus;
	bool mCompressed;
	static const CGlob mEnding[ENDINGSIZE];
	static const CGlob mName[NAMESIZE];
	static const CGlob mMagic0[MAGIC0SIZE];
	static const CGlob mOpenOffice[OPENOFFICESIZE];
	static const CGlob mKOffice[KOFFICESIZE];
	static const CMagInfo mMagInfo[MAGINFOCOUNT];
};

inline std::string CMagic::GetSuffix()
{
	std::string ret;
	std::string::size_type pos = mPath.rfind('.');
	if (pos != 0 && pos != std::string::npos && mPath[pos - 1] != '/') {
		while (++pos < mPath.size()) ret += ((char)tolower(mPath[pos]));
	}
	return ret;
}

inline const char* CMagic::GetDescription(TMagic type)
{
	type = (TMagic)(type & UNCOMPRESSED);
	return cxxx::lib::cxgettext(mMagInfo[type].mDescription);
}

inline const char* CMagic::GetMimeType(TMagic type)
{
	type = (TMagic)(type & UNCOMPRESSED);
    return mMagInfo[type].mMime;
}

inline const char* CMagic::GetIcon(TMagic type)
{	
	type = (TMagic)(type & UNCOMPRESSED);
	return mMagInfo[type].mIcon;
}

inline bool CMagic::Less(const CGlob& g1, const CGlob& g2)
{
	return strcmp(g1.mPattern, g2.mPattern) < 0;
}

inline bool CMagic::LessMag(const CGlob& g1, const CGlob& g2)
{
	return memcmp(g1.mPattern + 1, g2.mPattern + 1, std::min(g1.mPattern[0], g2.mPattern[0])) < 0;
}

inline bool operator==(const CMagic::CGlob& g, const char* s)
{
	const char* t = g.mPattern;
	while (*t) if (*s++ != *t++) return false;
	return true;
}

#endif // CMagic_h
