/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CMessageTransceiver.h
  Info   : $Id: CMessageTransceiver.h,v 1.4 2005/02/16 20:22:41 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2005 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CMessageTransceiver_h
#define CMessageTransceiver_h

// cxxx headers
#include <cxxx/Cxxx.h>

// C++ headers
#include <cassert>
#include <vector>
#include <algorithm>
#include <deque>

enum
{
    ButtonDblClick = LASTEvent,
	CxxxMessage
};

class CMessageTransceiver
{
public:
	CMessageTransceiver(CMessageTransceiver* target, unsigned msgid);
	virtual ~CMessageTransceiver();
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
    virtual bool OnUpdate(unsigned msgid) const;
	void SendMessage(TMessage type, unsigned value = 0);
    void PostMessage(TMessage type, unsigned value = 0);
	bool GetUpdate();
	void RequestIdle(bool req);
	Window GetXID();
	void SetMessageTarget(CMessageTransceiver* target);
	void SetMessageID(unsigned msgid);
protected:
	void PushEvent(const XEvent& e);
	XEvent PopEvent();
	bool EventPending();
	bool EventsQueued();
	Window mXID;
private:
	static std::deque<XEvent> mEventQueue;
	unsigned mMsgID;
	CMessageTransceiver* mTarget;
};

inline void CMessageTransceiver::SendMessage(TMessage type, unsigned value)
{
	if (mTarget != NULL) mTarget->OnMessage(mMsgID, type, value);
}

inline bool CMessageTransceiver::GetUpdate()
{
	if(mTarget != NULL) return mTarget->OnUpdate(mMsgID);
	return false;
}

inline bool CMessageTransceiver::EventPending()
{
    return (!mEventQueue.empty() || XPending(cxxx::cxDpy));
}

inline bool CMessageTransceiver::EventsQueued()
{
	return (!mEventQueue.empty());
}

inline void CMessageTransceiver::PostMessage(TMessage type, unsigned value)
{
	if (mTarget == NULL) return;
	XEvent e;
	while (XPending(cxxx::cxDpy)) {
       	XNextEvent(cxxx::cxDpy, &e);
		PushEvent(e);
	}
	e.xclient.type = CxxxMessage;
	e.xclient.send_event = True;
	e.xclient.display = cxxx::cxDpy;
	e.xclient.message_type = type;
	e.xclient.window = mTarget->GetXID();
	e.xclient.format = 32;
	e.xclient.data.l[0] = mMsgID;
	e.xclient.data.l[1] = value;
	// do not pile up scroll messages
	if (type == SCROLL || type == MOTION) {
		for (std::deque<XEvent>::iterator i = mEventQueue.begin(); i != mEventQueue.end(); ++i) {
			if (i->xclient.message_type == (unsigned)type && i->xclient.data.l[0] == (int)mMsgID) {
				mEventQueue.erase(i);
				break;
			}
		}
	}
	PushEvent(e);
}

inline void CMessageTransceiver::PushEvent(const XEvent& e)
{
	if (e.type == ConfigureNotify) {
		for (std::deque<XEvent>::iterator i = mEventQueue.begin(); i != mEventQueue.end(); ++i) {
			if (i->type == ConfigureNotify && i->xconfigure.window == e.xconfigure.window) {
				mEventQueue.erase(i);
				break;
			}
		}
	}
	mEventQueue.push_back(e);
}

inline XEvent CMessageTransceiver::PopEvent()
{
	assert (EventsQueued());
	XEvent e = mEventQueue.front();
	mEventQueue.pop_front();
	return e;
}

inline Window CMessageTransceiver::GetXID()
{
	return mXID;
}

inline void CMessageTransceiver::SetMessageTarget(CMessageTransceiver* target)
{
	mTarget = target;
}

inline void CMessageTransceiver::SetMessageID(unsigned msgid)
{
	mMsgID = msgid;
}

#endif // CMessageTransceiver_h



