/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CNumEdit.h
  Info   : $Id: CNumEdit.h,v 1.8 2006/01/21 14:26:21 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005, 2006 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CNumEdit_h
#define CNumEdit_h

// cxxx headers
#include <cxxx/CEdit.h>
#include <cxxx/CBuddy.h>

// C++ headers
#include <sstream>

class CNumEdit : public CEdit
{
private:
	enum
	{
		IDBUDDY = cxxx::msgid::MSGID_CNumEdit,
		BORDER_WIDTH = 2,
		WIDTH = 50,
		HEIGHT = 20
	};
public:
    CNumEdit(CWindow* parent, unsigned msgid, bool show = true);
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnConfigure(XConfigureEvent& e);
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
	virtual void OnMouseWheelUp(XButtonEvent& e);
	virtual void OnMouseWheelDown(XButtonEvent& e);
	virtual void OnKey(unsigned key, unsigned mask);
	virtual ~CNumEdit();
	void SetValue(int value);
	int GetValue();
	void SetRange(int min, int max);
	void SetStep(int step);
protected:
	void DisplayValue(int value);
	CBuddy mBuddy;
};

inline void CNumEdit::SetValue(int value)
{
	mBuddy.SetPosition(value);
	DisplayValue(value);
}

inline int CNumEdit::GetValue()
{
	return mBuddy.GetPosition();
}

inline void CNumEdit::DisplayValue(int value)
{
	std::ostringstream os;
	os << value;
	SetText(os.str());
}

inline void CNumEdit::SetRange(int min, int max)
{
	mBuddy.SetRange(min, max);
}

inline void CNumEdit::SetStep(int step)
{
	mBuddy.SetStep(step);
}

#endif // CNumEdit_h
