/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CProfile.h
  Info   : $Id: CProfile.h,v 1.5 2005/11/27 17:53:06 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2005 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CProfile_h
#define CProfile_h

// cxxx headers
#include <cxxx/Cxxx.h>

// C++ headers
#include <vector>

class CProfile
{
public:
    typedef std::vector<std::string> TContainer;
	CProfile();
    CProfile(const std::string& file);
	~CProfile();
	unsigned Read(const std::string& entry, unsigned def);
	std::string Read(const std::string& entry, const std::string& def);
	void Write(const std::string& entry, unsigned val);
	void Write(const std::string& entry, const std::string& val);
	const std::string& GetFileName() const;
	void SetFileName(const std::string& file);
	void ReadFile(const std::string& file);
    bool IsValid();
	void Flush();
protected:
	std::string GetValue(const std::string& s);
	TContainer::iterator FindEntry(const std::string& entry);
	std::string mFile;
	TContainer mArray;
	bool mChanged;
};

inline const std::string& CProfile::GetFileName() const
{
	return mFile;
}

inline void CProfile::SetFileName(const std::string& file)
{
	mFile = file;
}

inline bool CProfile::IsValid()
{
    if (mFile.empty()) return false;
    return !mArray.empty();
}
#endif // CProfile_h
