/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CScrollBar.h
  Info   : $Id: CScrollBar.h,v 1.4 2006/01/15 16:26:29 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2006 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CScrollBar_h
#define CScrollBar_h

// cxxx headers
#include <cxxx/CWindow.h>

class CScrollBar : public CWindow
{
	enum
	{
		IDTIMER = cxxx::msgid::MSGID_CScrollBar
	};

public:
	CScrollBar(CWindow* parent, unsigned msgid, bool show = true);
    virtual void OnExpose(XExposeEvent& e) = 0;
	virtual void OnLButtonDown(XButtonEvent& e) = 0;
	virtual void OnButtonUp(XButtonEvent& e) = 0;
	virtual void OnMouseWheelUp(XButtonEvent& e);
	virtual void OnMouseWheelDown(XButtonEvent& e);
	virtual void OnEvent(XEvent& e) = 0;
	virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);

    void SetPage(int page)
    {
        mPage = page;
    }

	void SetRange(int range)
    {
        mRange = range;
    }

	void SetPosition(int pos)
    {
	    // perform some rangecheck
	    if (pos + mPage > mRange)	pos = mRange - mPage;
	    if (pos < 0) pos = 0;
		if (mPosition == pos) return;
        mPosition = pos;
        if (IsEnabled()) PostMessage(SCROLL, mPosition);
		DrawTrackBar();
    }

    int GetPosition()
    {
        return mPosition;
    }

	void Enable(bool enable)
    {
        if (IsEnabled() == enable) return ;
    	CWindow::Enable(enable);
	    if (enable) Redraw();
    	else mPosition = 0;
    }

protected:
	virtual int CalculatePosition(int x) = 0;
	virtual void DrawTrackBar() = 0;
    void DrawButton(int x, int y, int wh, bool pressed, const CIcon& icon);

    void PosDec()
    {
        if (mPosition > 0) SetPosition(mPosition - 1);
    }

    void PosInc()
    {
	    if (mPosition < mRange) SetPosition(mPosition + 1);
    }

	void PageDec()
    {
	    int pos = mPosition - mPage;
        if (pos < 0) pos = 0;
   		if (pos != mPosition) SetPosition(pos);
    }

	void PageInc()
    {
    	int pos = mPosition + mPage;
        if (pos > mRange) pos = mRange;
	    if (pos != mPosition) SetPosition(pos);
    }

    int mPage;
	int mRange;
	int mPosition;
	int mTBStart;
	int mTBEnd;
	CIcon mArrow1;
	CIcon mArrow2;
	CTimer mTimer;
	bool mState1;
	bool mState2;
	bool mTracking;
	bool mEnabled;
	int mOffset;
};

class CHScrollBar : public CScrollBar
{
public:
	CHScrollBar(CWindow* parent, unsigned msgid, bool show = true);
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnButtonUp(XButtonEvent& e);
	virtual void OnEvent(XEvent& e);
protected:
	virtual int CalculatePosition(int x)
    {
	    int width = mWidth - 2 * mHeight + mTBStart - mTBEnd;
	    if (!width) return 0;
	    int pos = (x * (mRange - mPage) + width / 2) / width;
	    if (pos < 0) pos = 0;
	    if (pos > mRange - mPage) pos = mRange - mPage;
	    return pos;
     }
	 virtual void DrawTrackBar();
};

class CVScrollBar : public CScrollBar
{
public:
	CVScrollBar(CWindow* parent, unsigned msgid, bool show = true);
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnButtonUp(XButtonEvent& e);
	virtual void OnEvent(XEvent& e);
protected:
	virtual int CalculatePosition(int y)
    {
	    int height = mHeight - 2 * mWidth + mTBStart - mTBEnd;
	    if (!height) return 0;
	    int pos = (y * (mRange - mPage) + height / 2) / height;
	    if (pos < 0) pos = 0;
	    if (pos > mRange - mPage) pos = mRange - mPage;
	    return pos;
     }
	 virtual void DrawTrackBar();
};

#endif // CScrollBar_h
