/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CSelection.h
  Info   : $Id: CSelection.h,v 1.5 2007/06/07 22:48:22 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2003 - 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CSelection_h
#define CSelection_h

// cxxx headers
#include <cxxx/CTimer.h>

// forward declarations
class CWindow;

class CSelection : public CMessageTransceiver
{
	enum
	{
		IDTIMER = cxxx::msgid::MSGID_CSelection
	};

    struct CSelectionItem
    {
        Atom mName;
        Time mStartTime;
        Time mEndTime;
        CWindow* mOwner;
    };
public:
	CSelection();
    ~CSelection();
    virtual void OnMessage(unsigned msgid, TMessage type, unsigned value);
    bool Grab(CWindow* w, Atom sel, Time time);
    void Release(Atom sel, Time time);
    void Release(CWindow*);
    CWindow* GetOwner(Atom sel, Time time);
    void HandleRequest(XSelectionRequestEvent& e);
    void HandleClear(XSelectionClearEvent& e);
    void HandleNotify(XSelectionEvent& e);
    void HandleProperty(XPropertyEvent& e)
    {
        mPropertyNotify = e;
    }
    void CancelRequest(XSelectionRequestEvent& e);
    void ConfirmRequest(XSelectionRequestEvent& e);
    bool Request(CWindow* req, Atom sel, Atom typ, Atom prop, std::vector<unsigned char>& ret);
    bool Init(const std::string& cxxxdir);
	const std::string& GetClipSelDirectory();
	const std::string& GetDnDSelDirectory();
    static Atom TARGETS;
    static Atom INCR;
    static Atom MULTIPLE;
    static Atom TIMESTAMP;
    static Atom CLIPBOARD;
protected:
    bool OnRequest(CWindow* owner, XSelectionRequestEvent& e);
private:
    std::vector<CSelectionItem> mSelectionList;
    CTimer mTimer;
    bool mTimeout;
    bool mLocked;
    XSelectionEvent mSelectionNotify;
    XPropertyEvent mPropertyNotify;
	std::string mClipSelDir;
	std::string mDnDSelDir;
};

inline const std::string& CSelection::GetClipSelDirectory()
{
	return mClipSelDir;
}

inline const std::string& CSelection::GetDnDSelDirectory()
{
	return mDnDSelDir;
}

#endif // CSelection_h
