/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CSlider.h
  Info   : $Id: CSlider.h,v 1.7 2006/01/28 14:36:16 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2005, 2006 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CSlider_h
#define CSlider_h

// cxxx headers
#include <cxxx/CWindow.h>

// C++ headers
#include <cassert>

class CSlider : public CWindow
{
	enum
	{
		WIDTH = 120,
		HEIGHT = 20
	};		
public:
	CSlider(CWindow* parent, unsigned msgid, bool show = true);
    virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnButtonUp(XButtonEvent& e);
	virtual void OnMouseWheelUp(XButtonEvent& e);
	virtual void OnMouseWheelDown(XButtonEvent& e);
	virtual void OnEvent(XEvent& e);
	void SetRange(int min, int max);
	void SetPage(int page);
	void SetStep(int step);
	void SetPosition(int pos);
    int GetPosition();
	int GetStep();
	void Enable(bool enable);
protected:
	void PageDec();
	void PageInc();
	void DrawTrackBar();
	int CalculatePosition(int x);
	int mTBStart;
	int mPage;
	int mMin;
	int mMax;
	int mPosition;
	bool mTracking;
	int mOffset;
	int mStep;
};

inline void CSlider::SetRange(int min, int max)
{
	mMin = std::min(min, max);
	mMax = std::max(min, max);
}

inline void CSlider::SetPage(int page)
{
	assert(page < (mMax - mMin));
	mPage = page;
}

inline void CSlider::SetPosition(int pos)
{
	if (pos > mMax)	pos = mMax;
	if (pos < mMin) pos = mMin;
	pos = ((2 * pos + mStep) / (2 * mStep)) * mStep;
	if (mPosition == pos) return;
	mPosition = pos;
	DrawTrackBar();
}

inline int CSlider::GetPosition()
{
	return mPosition;
}

inline void CSlider::SetStep(int step)
{
	assert(step >= 1);
	mStep = step;
}

inline int CSlider::GetStep()
{
	return mStep;
}

inline void CSlider::Enable(bool enable)
{
	if (IsEnabled() == enable) return ;
	CWindow::Enable(enable);
	if (enable) Redraw();
	else mPosition = 0;
}

inline void CSlider::PageDec()
{
	SetPosition(mPosition - mPage);
	if (IsEnabled()) PostMessage(SCROLL, mPosition);
}

inline void CSlider::PageInc()
{
	SetPosition(mPosition + mPage);
	if (IsEnabled()) PostMessage(SCROLL, mPosition);
}

inline int CSlider::CalculatePosition(int x)
{
	int width = mWidth - 24;
	int pos = (mMax - mMin) * (x - 2);
	pos += (mMax - mMin) / 2;
	return pos / width + mMin;
}

#endif // CSlider_h
