/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CStatusBar.h
  Info   : $Id: CStatusBar.h,v 1.4 2005/02/26 18:40:25 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2005 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CStatusBar_h
#define CStatusBar_h

// cxxx headers
#include <cxxx/CWindow.h>

class CStatusField
{
public:
	CStatusField(char pos, int width);

	std::string mText;
	bool mLeftAlign;
	int mDefaultWidth;
	int mPos;
	unsigned mWidth;
};	

class CStatusBar : public CWindow
{
	enum 
	{
		BORDER_WIDTH = 2,
		TEXT_BORDER = 2,
		HEIGHT = 22
	};
public:
	CStatusBar(CWindow* parent, bool show = true);
	~CStatusBar();
	virtual void OnConfigure(XConfigureEvent& e);
	virtual void OnExpose(XExposeEvent& e);
	unsigned AddField(char pos, int width);
	void SetText(unsigned idx, const std::string& text);
	void DrawText(unsigned idx);
private:
	std::vector<CStatusField> mFields;
};

inline CStatusField::CStatusField(char pos, int width)
: mDefaultWidth(width),
  mPos(0),
  mWidth(0)
{
	if (pos == 'L' || pos == 'l') mLeftAlign = true;
	else mLeftAlign = false;
}
	
inline unsigned CStatusBar::AddField(char pos, int width)
{
	mFields.push_back(CStatusField(pos, width));
	return mFields.size();
}

inline void CStatusBar::SetText(unsigned idx, const std::string& text)
{
	assert(idx < mFields.size());
	if (mFields[idx].mText != text) {
		mFields[idx].mText = text;
		DrawText(idx);
	}
}

#endif // CStatusBar_h
