/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CTimer.h
  Info   : $Id: CTimer.h,v 1.4 2005/02/16 20:22:59 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2005 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CTimer_h
#define CTimer_h

// cxxx headers
#include <cxxx/CMessageTransceiver.h>

// C++ headers
#include <cassert>

// C headers
#include <sys/time.h>

class CTimer : public CMessageTransceiver
{
public:
	CTimer(CMessageTransceiver* target, unsigned msgid);
	~CTimer();
	void Stop();
	void SetTime(unsigned time);
	void SendMessage();
    static void Refresh(timeval& t);
protected:
    int TimeLeft(const timeval& now);
    static timeval Now();
	timeval mTimeOut;
    static std::vector<CTimer*> mTimerList;
public:
    // functor
    class TestTimer
    {
        timeval mNow;
    public:
        TestTimer()
        : mNow(Now())
        {
        }

        void operator()(CTimer* t)
        {
            if (t->TimeLeft(mNow) <= 0) {
                t->SendMessage();
            } else {
                mTimerList.push_back(t);
            }
        }
    };
    friend class CTimer::TestTimer;
};

inline void CTimer::Stop()
{
    if (mTimerList.empty()) return;
    std::vector<CTimer*>::iterator i = find(mTimerList.begin(), mTimerList.end(), this);
    if (i != mTimerList.end()) mTimerList.erase(i);
}

inline void CTimer::SetTime(unsigned tim)
{
    mTimeOut = Now();
    mTimeOut.tv_sec += tim / 1000;
    mTimeOut.tv_usec += (tim % 1000) * 1000;
    if(mTimeOut.tv_usec >= 1000000) {
        mTimeOut.tv_usec -= 1000000;
        ++mTimeOut.tv_sec;
    }
   	std::vector<CTimer*>::iterator i = find(mTimerList.begin(), mTimerList.end(), this);
    if(i == mTimerList.end()) mTimerList.push_back(this);
}

inline void CTimer:: SendMessage()
{
	CMessageTransceiver::SendMessage(TIMER, 0);
}

inline int CTimer::TimeLeft(const timeval& now)
{
    int sec = mTimeOut.tv_sec - now.tv_sec;
    int usec = mTimeOut.tv_usec - now.tv_usec;
    return sec * 1000 + usec / 1000;
}

inline timeval CTimer::Now()
{
    timeval now;
    gettimeofday(&now, NULL);
    return now;
}

#endif // CTimer_h



