/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CTopLevelWindow.h
  Info   : $Id: CTopLevelWindow.h,v 1.10 2006/09/10 15:39:03 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2006 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CTopLevelWindow_h
#define CTopLevelWindow_h

// cxxx headers
#include <cxxx/CWindow.h>

// forward declarations
class CDnDWindow;

class CTopLevelWindow : public CWindow
{
protected:
	enum
	{
		NO_NET = 1 << 0,
		NET_WM_NAME = 1 << 1,
		NET_WM_ICON_NAME = 1 << 2,
		NET_WM_ICON = 1 << 3,
		NET_WM_PID = 1 << 4,
		NET_WM_PING = 1 << 5,
		NET_SUPPORTING_WM_CHECK = 1 << 6
	};
public:
	CTopLevelWindow(bool show = true);
	CTopLevelWindow(unsigned msgid, bool show = true);
	void Init(bool show);
	virtual ~CTopLevelWindow();
	virtual void OnEvent(XEvent& e);
	virtual void OnClientMessage(XClientMessageEvent& e);
	virtual void OnQuit();
	virtual void HandleKeyPress(XKeyEvent& e);
	virtual void OnKey(unsigned key, unsigned state);
	virtual void SetFocusWindow(CWindow* w);
	virtual void OnFocusEvent(XFocusChangeEvent& e);

    virtual void OnShortcut(unsigned key, unsigned state)
	{
    }
	CWindow* GetFocusWindow();
	void SetCaption(const std::string& c);
	void FixSize();
	void SetMinSize(int w, int h);
	void SetMaxSize(int w, int h);
	void TransientFor(CTopLevelWindow* w);
	void BindKey(unsigned key, CWindow* w);
	void ReleaseKey(unsigned key);
	void ReleaseKey(CWindow* w);
	unsigned DoModal();
	bool IsRunning();
    void OnDestroy(const CWindow* w);
    void OnDestroy(const CDnDWindow* w);
    void SendClientEvent(XClientMessageEvent& e, Window target = None)
    {
        e.type = ClientMessage;
        e.display = cxxx::cxDpy;
        if (target == None) target = e.window;
        assert(target != None);
        if (target == GetXID()) OnClientMessage(e);
        else XSendEvent(cxxx::cxDpy, target, False, NoEventMask, (XEvent*)&e);
    }
	void ResetStartupNotification(const std::string& id);
protected:
	bool CreateWMPixmaps();
	bool CreateNetWMPixmaps();
	bool CreateIcccmPixmaps();
	unsigned GetWMCaps();
protected:
	CWindow* mAccelTable['Z' - 'A' + 1];
	unsigned mReturn;
	bool mShutDown;
	CWindow* mFocusWindow;
	CWindow* mKeyFocusWindow;
	CTopLevelWindow* mTransient;
	CTopLevelWindow* mTransientFor;
    static Atom WM_PROTOCOLS_ATOM;
    static Atom WM_DELETE_ATOM;
	static Atom WM_ICON_SIZE_ATOM;
	static Atom NET_SUPPORTED_ATOM;
	static Atom NET_WM_NAME_ATOM;
	static Atom NET_WM_ICON_NAME_ATOM;
	static Atom NET_WM_ICON_ATOM;
	static Atom NET_WM_PID_ATOM;
	static Atom NET_WM_PING_ATOM;
	static Atom NET_SUPPORTING_WM_CHECK_ATOM;
	static Pixmap mWMIcon;
	static Pixmap mWMIconMask;
    static unsigned mWMCaps;
};

inline void CTopLevelWindow::BindKey(unsigned key, CWindow *w)
{
	if (key < 'A' || key > 'Z') {
		cxxx::cxError << cxxx::lib::_("CTopLevelWindow::BindKey\nindex out of range", 2) << CError::Warn;
		return ;
	}
	if (w && mAccelTable[key - 'A']) {
		char s[2];
		*s = key;
		s[1] = 0;
		cxxx::cxError << cxxx::lib::_("Duplicate Accelerator {1}", 3) << s << CError::Warn;
	}
	mAccelTable[key - 'A'] = w;
}

inline void CTopLevelWindow::ReleaseKey(unsigned key)
{
	BindKey(key, NULL);
}

inline void CTopLevelWindow::ReleaseKey(CWindow* w)
{
    CWindow** end = mAccelTable + 'Z' - 'A' + 1;
	CWindow** i = std::find(mAccelTable, end, w);
    if (i < end) {
        (*i) = NULL;
        ReleaseKey(w);
	}
}

inline void CTopLevelWindow::TransientFor(CTopLevelWindow* w)
{
	if (w != this) {
		if (w->mTransient)
			w = w->mTransient;
		XSetTransientForHint(cxxx::cxDpy, GetXID(), cxxx::cxMainWin->GetXID());
		mTransientFor = w;
		w->mTransient = this;
	}
}

inline CWindow* CTopLevelWindow::GetFocusWindow()
{
	return mFocusWindow;
}

inline bool CTopLevelWindow::IsRunning()
{
	return !mShutDown;
}

inline bool CTopLevelWindow::CreateWMPixmaps()
{
	if ((mWMCaps & NET_WM_ICON) && CreateNetWMPixmaps()) return true;
	if (mWMIcon != None) return true;
	return CreateIcccmPixmaps();
}	

#endif // CTopLevelWindow_h
