/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : CTreeControl.h
  Info   : $Id: CTreeControl.h,v 1.14 2005/08/02 21:50:13 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2005 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef CTreeControl_h
#define CTreeControl_h

// cxxx headers
#include <cxxx/CControl.h>
#include <cxxx/CTreeItem.h>

// forward declarations

class CTreeControl : public CControl
{
protected:
	enum
	{
		INDENT_STEP = 15,
		ICON_BORDER = 2,
		TEXT_BORDER = 2
	};
public:
	CTreeControl(CWindow* parent, unsigned msgid, bool show = true);
	virtual void OnExpose(XExposeEvent& e);
	virtual void OnLButtonDown(XButtonEvent& e);
	virtual void OnRButtonDown(XButtonEvent& e);
	virtual void Open(unsigned idx);
	virtual void Close(unsigned idx);
	void SetTree(const std::string& closed, const std::string& opened, const std::string& text);
	void SetIconSize(unsigned w, unsigned h);
	virtual void Append(CTreeItem& parent, const std::string& closed, const std::string& opened, const std::string& text);
	CTreeItem& GetItem(unsigned idx);
protected:
	virtual void CalculateDocSize();
	virtual unsigned ItemFromPos(int x, int y);
	virtual void DrawItem(unsigned idx);
	virtual void ScrollIn(unsigned idx);
	unsigned GetListSize(CTreeItem& item) const;
	unsigned GetRow(int y) const;
	void Sync(CTreeItem& item);
	unsigned GetIndex(const CTreeItem& item) const;
	void GetBounds(int& x, int& y, unsigned& w, unsigned& h);
	CIcon mPlus;
	CIcon mMinus;
	CTreeItem mTree;
};

inline void CTreeControl::SetTree(const std::string& closed, const std::string& opened, const std::string& text)
{
	unsigned w = GetTextWidth(text) + mIconWidth + 2 * TEXT_BORDER;;
	mTree = CTreeItem(closed, opened, text, w, mIconHeight);
}

inline unsigned CTreeControl::GetIndex(const CTreeItem& item) const
{
	for (unsigned i = 0; i < mList.size(); ++i) if ((const CControlItem&)item == mList[i]) return i;
	return -1u;
}

inline unsigned CTreeControl::ItemFromPos(int x, int y)
{
	ScreenToWindow(x, y);
	unsigned idx = GetRow(y);
	if (idx >= mList.size()) return -1u;
	CTreeItem item = GetItem(idx);
	int tmp = item.GetIndentation() * INDENT_STEP + INDENT_STEP;
	if (x < tmp) return -1u;
	tmp += item.GetWidth();
	if(x > tmp) return -1u;
	return idx;
}

inline unsigned CTreeControl::GetRow(int y) const
{
	if (y < 0) return 0;
	unsigned r = y / mIconHeight;
	if (r >= mList.size()) r = -1u;
	return r;
}

inline void CTreeControl::SetIconSize(unsigned w, unsigned h)
{
	mIconWidth = w + 2 * ICON_BORDER;
	mIconHeight = h + 2 * ICON_BORDER;
}

inline void CTreeControl::ScrollIn(unsigned idx)
{
	int xi = GetItem(idx).GetIndentation() * INDENT_STEP + INDENT_STEP + GetItem(idx).GetWidth();
	int yi = idx * mIconHeight;
	int xw = BORDER_WIDTH + mViewWidth;
	int yw = BORDER_WIDTH;
	int scrlx = 0;
	int scrly = 0;
	WindowToScreen(xi, yi);
	if (yi < yw) scrly = (yi - yw - SCROLL_STEP + 1) / SCROLL_STEP;
	else {
		yi += mIconHeight;
		yw += mViewHeight;
		if (yi > yw) scrly = (yi - yw + SCROLL_STEP - 1) / SCROLL_STEP;
	}
	if (xi > xw) scrlx = (xi - xw + SCROLL_STEP - 1) / SCROLL_STEP;
	xi -= GetItem(idx).GetWidth() + INDENT_STEP + scrlx * SCROLL_STEP;
	xw -= mViewWidth;
	if (xi < xw) scrlx += (xi - xw - SCROLL_STEP + 1) / SCROLL_STEP;
	if (scrlx != 0) mHScrollBar.SetPosition(mHScrollBar.GetPosition() + scrlx);
	if (scrly != 0) mVScrollBar.SetPosition(mVScrollBar.GetPosition() + scrly);
}

inline CTreeItem& CTreeControl::GetItem(unsigned idx)
{
	assert(idx < mList.size());
	return (CTreeItem&)mList[idx];
}

#endif // CTreeControl_h
