/*----------------------------------------------------------------------------
  Project: cxxx <http://cxplorer.sourceforge.net>
  File   : Cxxx.h
  Info   : $Id: Cxxx.h,v 1.30 2007/05/18 09:11:07 gkraeuter Exp $
----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Copyright © 2002 - 2007 Gerhard Kräuter <gkraeuter@users.sourceforge.net>

  This  software  is  provided  'as-is',  without  any  express  or  implied
  warranty.  In  no  event  will  the authors be held liable for any damages
  arising from the use of this software.

  Permission  is  granted  to  anyone  to use this software for any purpose,
  including  commercial  applications,  and  to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The  origin  of  this software must not be misrepresented; you must not
     claim  that  you  wrote the original software. If you use this software
     in  a  product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

----------------------------------------------------------------------------*/

#ifndef Cxxx_h
#define Cxxx_h

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#ifdef HAVE_LIBFONTCONFIG
#ifdef HAVE_LIBFREETYPE
#ifdef HAVE_LIBXRENDER
#define AA_TEXT_SUPPORT
#endif // HAVE_LIBXRENDER
#endif // HAVE_LIBFREETYPE
#endif // HAVE_LIBFONTCONFIG

#define CXXX_KDE3_WORKAROUND

#include <X11/Xlib.h> // for Display
#include <X11/Xatom.h>

#include <cxxx/CError.h>
#include <cxxx/CI18n.h>
#include <vector> // if we do not have locale

#include <sys/stat.h> // for mode_t

#define _(String, index) cxgettext(index)

#ifndef XA_UTF8_STRING
 #define XA_UTF8_STRING cxxx::cxUtf8Atom
#endif // XA_UTF8_STRING
 
#ifndef cxxx_GET_FLAG
 #define cxxx_GET_FLAG(flag) (((flag) + 1) & ~(flag))
#endif


class CApplication;
class CDnDManager;
class CFileHandler;
class CFont;
class CProfile;
class CSelection;
class CTopLevelWindow;
class CGC;

//message types
typedef enum TMessage
{
	TIMER				= 1 << 0,
	IDLE				= 1 << 1,
	ERROR				= 1 << 2,
	ACCELERATOR			= 1 << 3,
	CHECK				= 1 << 4,
	UNCHECK				= 1 << 5,
	CONFIRM_SELECTION	= 1 << 6,
	SELECTION_CHANGED	= 1 << 7,
	SCROLL				= 1 << 8,
	PRESSED				= 1 << 9,
	RELEASED			= 1 << 10,
	MOTION				= 1 << 11
};

namespace cxxx
{
	namespace filesort
	{
		enum
		{
			NAME = 1 << 0,
			// CASE = 1 << 1,
			SIZE = 1 << 2,
			TIME = 1 << 3,
			TYPE = 1 << 4,
			ASCENDING = 1 << 6,
			DIRFIRST = 1 << 7,
			HIDDEN = 1 << 8
		};
	}

	namespace msgid 
	{
		enum {
			MSGID_Step = 0x100000,
			MSGID_CApplication		= MSGID_Step * 0x01,
			MSGID_CButton			= MSGID_Step * 0x02,
			MSGID_CCaret			= MSGID_Step * 0x03,
			MSGID_CCheckButton		= MSGID_Step * 0x03,
			MSGID_CCxxxAbout		= MSGID_Step * 0x04,
			MSGID_CDnDAskDialog		= MSGID_Step * 0x05,
			MSGID_CDnDManager		= MSGID_Step * 0x06,
			MSGID_CDnDWindow		= MSGID_Step * 0x07,
			MSGID_CDropDownList	 	= MSGID_Step * 0x08,
			MSGID_CDropDownPopup	= MSGID_Step * 0x09,
			MSGID_CDupFNDialog		= MSGID_Step * 0x0a,
			MSGID_CEdit				= MSGID_Step * 0x0b,
			MSGID_CFileListControl	= MSGID_Step * 0x0c,
			MSGID_CFileTree			= MSGID_Step * 0x0d,
			MSGID_CHSplitterWindow	= MSGID_Step * 0x0e,
			MSGID_CLabel			= MSGID_Step * 0x0f,
			MSGID_CListControl		= MSGID_Step * 0x10,
			MSGID_CMenuBar			= MSGID_Step * 0x11,
			MSGID_CMenuItem			= MSGID_Step * 0x12,
			MSGID_CMessageBox		= MSGID_Step * 0x13,
			MSGID_CPopup			= MSGID_Step * 0x14,
			MSGID_CPropertyTab		= MSGID_Step * 0x15,
			MSGID_CPushButton		= MSGID_Step * 0x16,
			MSGID_CRadioButton		= MSGID_Step * 0x17,
			MSGID_CScrollBar		= MSGID_Step * 0x18,
			MSGID_CSelection		= MSGID_Step * 0x19,
			MSGID_CStatusBar		= MSGID_Step * 0x1a,
			MSGID_CTimer			= MSGID_Step * 0x1b,
			MSGID_CToolButton		= MSGID_Step * 0x1c,
			MSGID_CToolTip			= MSGID_Step * 0x1d,
			MSGID_CTopLevelWindow	= MSGID_Step * 0x1e,
			MSGID_CTreeControl		= MSGID_Step * 0x1f,
			MSGID_CTreeView			= MSGID_Step * 0x20,
			MSGID_CWindow			= MSGID_Step * 0x21,
			MSGID_CControl			= MSGID_Step * 0x22,
			MSGID_CFileOpenDialog	= MSGID_Step * 0x23,
			MSGID_CColorDialog		= MSGID_Step * 0x24,
			MSGID_CNumEdit			= MSGID_Step * 0x25,
			MSGID_CBuddy			= MSGID_Step * 0x26,
			MSGID_CSettings			= MSGID_Step * 0x27,
			MSGID_CTimingSheet		= MSGID_Step * 0x28,
			MSGID_CColorSheet		= MSGID_Step * 0x29,
			MSGID_CFontSheet		= MSGID_Step * 0x2a,
			MSGID_Last				= MSGID_Step * 0x2b
		};
	}
	enum
	{
		S_IFLOCKED = cxxx_GET_FLAG(S_IFMT | S_IRWXU | S_IRWXG | S_IRWXO | S_ISUID | S_ISGID | S_ISVTX),
		S_IFLINKED = cxxx_GET_FLAG(S_IFMT | S_IRWXU | S_IRWXG | S_IRWXO | S_ISUID | S_ISGID | S_ISVTX | S_IFLOCKED)
	};
	// essentials
	extern Display*         cxDpy;
	extern CApplication*    cxApp;
	extern CTopLevelWindow* cxMainWin;
	// fonts
	extern CFont*           cxTextFont;
	extern CFont*           cxTitleFont;
	extern Atom				cxUtf8Atom;
	// helpers
	extern CI18n			cxCxxxIntl;
	extern CI18n			cxAppIntl;
	extern CDnDManager*		cxDnDManager;
	extern CError			cxError;
	extern CFileHandler*	cxFileManager;
	extern CGC*				cxGCManager;
	extern CSelection*		cxSelManager;
	// settings
	extern CProfile*        cxProfile;
	namespace lib
	{
		extern CProfile*	cxProfile;
	}
	extern std::locale 		cxLocale;
	extern std::vector<std::string> cxIconDir;
	extern std::string		cxHomeDir;
	extern unsigned char    cxBitSwap[256];
	extern const char*      cxVersion;
	extern bool				cxUseUtf8;
	/*** internationalisation ***/
	namespace lib
	{
		inline const char* cxgettext(unsigned idx)
		{
			return ::cxxx::cxCxxxIntl.GetText(idx);
		}
	}

	namespace noop
	{
		inline unsigned cxgettext(unsigned idx)
		{
			return idx;
		}
	}	

	inline const char* cxgettext(unsigned idx)
	{
		return ::cxxx::cxAppIntl.GetText(idx);
	}

	const char Utf8toLatinChar(const std::string& s, unsigned& idx);

	inline std::string Utf8ToLatin(const std::string& s)
	{
		if (cxUseUtf8) {
			std::string ret;
			for (unsigned i = 0; i < s.size(); ++i) {
				if ((unsigned char)s[i] > 0x80) ret += Utf8toLatinChar(s, i);
				else ret += s[i];
			}
			return ret;
		} else return s;
	}

	inline std::string LatinToUtf8(const std::string& s)
	{
		if (cxUseUtf8) {
			std::string ret;
			for (unsigned i = 0; i < s.size(); ++i) {
				if ((unsigned char)s[i] > 0x80) {
					ret += ((s[i] >> 6) & 3) | '\xc0';
					ret += s[i] & '\xbf';
				} else ret += s[i];
			}
			return ret;
		} else return s;
	}

	inline bool S_ISLINKED(mode_t mode)
	{
		return (mode & S_IFLINKED) != 0;
	}
		
	inline bool S_ISLOCKED(mode_t mode)
	{
		return (mode & S_IFLOCKED) != 0;
	}
}

#endif // Cxxx_h
